/*
	blockdev.h
	Simple functionality for startup, mounting and unmounting of raw devices.
	
 Copyright (c) 2007 SukkoPera, based on work by Michael "Chishm" Chisholm.
	
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright notice,
     this list of conditions and the following disclaimer in the documentation and/or
     other materials provided with the distribution.
  3. The name of the author may not be used to endorse or promote products derived
     from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	2007-01-08 - SukkoPera
		* Original release
*/


#ifndef _LIBBLOCKDEV_H
#define _LIBBLOCKDEV_H

#ifdef __cplusplus
extern "C" {
#endif

// When compiling for NDS, make sure NDS is defined
#ifndef NDS
 #if defined ARM9 || defined ARM7
  #define NDS
 #endif
#endif

#ifdef NDS
 #include <nds/jtypes.h>
#else
 #include "gba_types.h"
#endif

#ifndef _LIBFAT_H	/* Only define this if we are not using libfat */
#ifdef NDS
typedef enum {PI_DEFAULT, PI_SLOT_1, PI_SLOT_2, PI_CUSTOM} PARTITION_INTERFACE;
#else
typedef enum {PI_CART_SLOT} PARTITION_INTERFACE;
#endif
#endif

struct IO_INTERFACE_STRUCT ;

/*
Initialise any inserted block-devices.
Add the raw device driver to the devoptab, making it available for standard file functions.
setAsDefaultDevice: if true, make this the default device driver for file operations
*/
bool rawioInit (bool setAsDefaultDevice);

/*
Calls fatInit with setAsDefaultDevice = true.
*/
bool rawioInitDefault (void);

/*
Mount the device specified by partitionDevice
PD_DEFAULT is not allowed, use _FAT_partition_setDefaultDevice
PD_CUSTOM is not allowed, use _FAT_partition_mountCustomDevice
*/
bool rawioMountNormalInterface (PARTITION_INTERFACE partitionNumber);

#if 0
/*
Mount a partition on a custom device
*/
bool _RAWIO_partition_mountCustomInterface (const IO_INTERFACE* device);
#endif

/*
Unmount the partition specified by partitionNumber
If there are open files, it will fail
*/
bool rawioUnmount (PARTITION_INTERFACE partitionNumber);

/*
Forcibly unmount the partition specified by partitionNumber
Any open files on the partition will become invalid
The cache will be invalidated, and any unflushed writes will be lost
*/
bool rawioUnsafeUnmount (PARTITION_INTERFACE partitionNumber);

#ifdef __cplusplus
}
#endif
#endif // _LIBBLOCKDEV_H
